/* 
Name: BTFtools.h
Version: 1.0

Description: Additional functions for BTFBASE loading into texture rep.
Author: Jiri Filip, UTIA AS CR, filipj'at'utia'dot'cas'dot'cz
Date: 28/05/07

COPYRIGHT ======================================================================

This source code is release under Creative Commons licence
(Attribution-NonCommercial 3.0 Unported) specified here:

http://creativecommons.org/licenses/by-nc/3.0/
http://creativecommons.org/licenses/by-nc/3.0/legalcode

You are free:
    * to Share - to copy, distribute and transmit the work
    * to Remix - to adapt the work
Under the following conditions:
    * Attribution - You must attribute the work in the manner specified by the
      author or licensor (but not in any way that suggests that they endorse you
      or your use of the work).

      Attribute this work:
      ------------------------------------------------
      Bidirectional Texture Function Compression based on Multi-Level Vector 
      Quantization
      as the paper written by
          Vlastimil Havran 	Jiri Filip 	Karol Myszkowski

      and published in Computer Graphics Forum, Volume 29, Issue 1, 
      pages 175-190, March 2010, DOI: 10.1111/j.1467-8659.2009.01585.x

      Project page:
        http://www.cgg.cvut.cz/members/havran/btfbase/
      ------------------------------------------------
     
    * Noncommercial - You may not use this work for commercial purposes.

With the understanding that:
    * Waiver - Any of the above conditions can be waived if you get permission 
      from the copyright holder.
    * Public Domain - Where the work or any of its elements is in the public 
      domain under applicable law, that status is in no way affected by 
      the license.
    * Other Rights - In no way are any of the following rights affected by the 
      license:
          o Your fair dealing or fair use rights, or other applicable copyright 
            exceptions and limitations;
          o The author's moral rights;
          o Rights other persons may have either in the work itself or in how 
            the work is used, such as publicity or privacy rights.
    * Notice - For any reuse or distribution, you must make clear to others 
         the license terms of this work. The best way to do this is with a link 
         to this web page.
===========================================================================  */

int invertMatrix(float* mat, float* im);
void vec_diff(float *x, float *y, float *res);
int vec_norm(float *x);
void vec_crossprod(float *x, float *y, float *res);

class BTFtools{
 private:
  char *matName;
  char *iPath;

  float stepBeta;    
  float stepAlpha;   
  float stepPhi;     
  float stepTheta;   

  int SlicesPerPhi;
  int SlicesPerTheta;
  int SlicesPerHemi; 
  int LengthOfSlice; 

  int texSize;

 public:
  //! integer 16bit parametric textures
  unsigned short *TI16_1;
  unsigned short *TI16_2;
  unsigned short *TI16_3;
  unsigned short *TI16_4;
  //! float 32bit parametric textures
  float *TF32_scales;
  float *TF32_data;

  BTFtools::BTFtools(int texSize, int SlicesPerPhi, int SlicesPerTheta, int SlicesPerHemi, int LengthOfSlice);
  ~BTFtools();
  void deallocTextures();

  //! \brief loads material BTF sample parameters into textures
  void setMaterial(char *iPath, float &BTFwidth, float &BTFheight, float &hdrMultiplier);
  //! \brief loads code-book file with indices
  void loadCodeBookIndex(char *indicesName, int sizeX, int valSizeX, unsigned short *indices, int indexStartPlane);
  //! \brief loads code-book file with scales
  void loadCodeBookScale(char *scalesName, int sizeX, float *scales, int scalePlane);
  //! \brief loads code-book file with indices and scales
  void loadCodeBookIndexScale(char *indicesName, char *scalesName, int sizeX, int valSizeX, unsigned short *indices, int indexStartPlane, float *scales, int scalePlane);
  //! \brief computes approprite UV index for parameters placement in the texture
  void putIndexAtUV(int iY, int iX, int val, int sizeX, int valSizeX, int &Y, int &X, int &valY, int &valX);
  //! \brief computes approprite UV index for parameters placement in the texture
  void putIndexAtUV(int iY, int iX, int sizeX, int &Y, int &X);
};
