/*
Name: TAlloc.h
Version: 1.0

Description: Source file with functions for data allocations and deallocations
Author: Jiri Filip, UTIA AS CR, filipj'at'utia'dot'cas'dot'cz
Date: 28/05/07
 */

#include <stdio.h>
#include <stdlib.h> 

//################################################################################
//# ALLOCATIONS                                                                 #
//################################################################################

float* allocation1(int nrl,int nrh){
  //! memory allocation, float vector [nrl..nrh] 
  float *m;
  if(nrh-nrl+1<=0) return NULL;

  m=(float*)calloc((unsigned)(nrh-nrl+1),sizeof(float));
  m -=nrl;

  return m;
} // allocation1 --------------------------------------------- 

int* iallocation1(int nrl,int nrh){
  //! memory allocation, float vector [nrl..nrh] 
  int *m;
  if(nrh-nrl+1<=0) return NULL;

  m=(int*)calloc((unsigned)(nrh-nrl+1),sizeof(int));
  m -=nrl;

  return m;
} // iallocation1 --------------------------------------------- 

unsigned short* sallocation1(int nrl,int nrh){
  //! memory allocation, unsigned short vector [nrl..nrh] 
  unsigned short *m;
  if(nrh-nrl+1<=0) return NULL;

  m=(unsigned short*)calloc((unsigned)(nrh-nrl+1),sizeof(unsigned short));
  m -=nrl;

  return m;
} // iallocation1 --------------------------------------------- 

char* callocation1(int nrl,int nrh){
  //! memory allocation, float vector [nrl..nrh] 
  char *m;
  if(nrh-nrl+1<=0) return NULL;

  m=(char*)calloc((unsigned)(nrh-nrl+1),sizeof(char));
  m -=nrl;

  return m;
} // callocation1 --------------------------------------------- 

unsigned char* ballocation1(int nrl,int nrh){
  //! memory allocation, float vector [nrl..nrh] 
  unsigned char *m;
  if(nrh-nrl+1<=0) return NULL;

  m=(unsigned char*)calloc((unsigned)(nrh-nrl+1),sizeof(unsigned char));
  m -=nrl;

  return m;
} // ballocation1 --------------------------------------------- 

float** allocation2(int nrl,int nrh,int ncl,int nch){
  //! memory allocation, float matrix [nrl..nrh][ncl..nch 
  int j;
  float **m;
  if((nrh-nrl+1<=0)||(nch-ncl+1<=0)) return NULL;

  m=(float**)calloc((unsigned)(nrh-nrl+1),sizeof(float*));
  if (!m) {printf(" allocation failure 1 in allocation2() \n");}
  m -=nrl;

  for(j=nrl;j<=nrh;j++){
    m[j]=(float*)calloc((unsigned)(nch-ncl+1),sizeof(float));
    if (!m[j]) {printf(" allocation failure 2 in allocation2() \n");}
    m[j] -=ncl;
  }
  return m;
} // allocation2 ----------------------------------------------- 

long** lallocation2(int nrl,int nrh,int ncl,int nch){
  //! memory allocation, float matrix [nrl..nrh][ncl..nch 
  int j;
  long **m;
  if((nrh-nrl+1<=0)||(nch-ncl+1<=0)) return NULL;

  m=(long**)calloc((unsigned)(nrh-nrl+1),sizeof(long*));
  if (!m) {printf(" allocation failure 1 in lallocation2() \n");}
  m -=nrl;

  for(j=nrl;j<=nrh;j++){
    m[j]=(long*)calloc((unsigned)(nch-ncl+1),sizeof(long));
    if (!m[j]) {printf(" allocation failure 2 in lallocation2() \n");}
    m[j] -=ncl;
  }
  return m;
} // lallocation2 ----------------------------------------------- 


//################################################################################
//# DEALLOCATIONS                                                                 #
//################################################################################

void freemem1(float *m,int nrl,int nrh){
  //! This method frees memory allocated by allocation1 
  nrh=nrh;
  free((float*)(m+nrl));
} // freemem1 ---------------------------------------------------- 

void freeimem1(int *m,int nrl,int nrh){
  //! This method frees memory allocated by allocation1 
  nrh=nrh;
  free((int*)(m+nrl));
} // freemem1 ---------------------------------------------------- 

void freesmem1(unsigned short *m,int nrl,int nrh){
  //! This method frees memory allocated by allocation1 
  nrh=nrh;
  free((unsigned short*)(m+nrl));
} // freemem1 ---------------------------------------------------- 

void freecmem1(char *m,int nrl,int nrh){
  //! This method frees memory allocated by allocation1 
  nrh=nrh;
  free((char*)(m+nrl));
} // freecmem1 ---------------------------------------------------- 

void freebmem1(unsigned char *m,int nrl,int nrh){
  //! This method frees memory allocated by allocation1 
  nrh=nrh;
  free((unsigned char*)(m+nrl));
} // freebmem1 ---------------------------------------------------- 

void freemem2(float **m,int nrl,int nrh,int ncl,int nch){
  //! frees memory allocated by allocation2 
  int i;
  //  nch=nch;
  //  nrh=nrh;

  for (i=nrh;i>=nrl;i--) free((float*)(m[i]+ncl));
  free((float**)(m+nrl));
} // freemem2 --------------------------------------------------- 

void freelmem2(long **m,int nrl,int nrh,int ncl,int nch){
  //! frees memory allocated by allocation2 
  int i;
  nch=nch;
  nrh=nrh;

  for (i=nrh;i>=nrl;i--) free((long*)(m[i]+ncl));
  free((long**)(m+nrl));
} // freelmem2 --------------------------------------------------- 
